/* The Last Ninja Remixed
 *  
 * Copyright (C) 2008-2009 Robert Crossfield
 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * 
 * $Id: actor.cpp 12 2009-04-10 04:08:13Z segra $
 * 
 */

#include "stdafx.h"

namespace NinjaRemixed {

	actor::actor(engine *Engine, word X, word Y, word Direction ) : engineObject( Engine ) {

		_action				= 0;
		_frameNumber		= 6;
		_graphicsReversed	= 0;
		_life				= 0x21;
		_positionX			= X;
		_positionY			= Y;
		_positionDirection	= Direction;

		_weapon				= 0;

	}

	actor::~actor() {

		if(_graphicsReversed)
			delete _graphicsReversed;

	}

	void actor::actionLoad( size_t actionNumber, size_t frames ) {

		// Load the animation for each direction
		for( size_t dir = 0; dir < 4; dir++)
			for( size_t x = 0; x < frames; x++)
				_animation.find( dir )->second->actionGet(actionNumber)->frameLoad( x );

	}

	void actor::directionLoad(  size_t frameSize, size_t direction, dword *actionTable, byte *bufferGraphics   ) {
		animationDirection *animDirection = (animationDirection*) new animationDirection( 32, 44, direction );

		animDirection->actionsLoad( frameSize,  actionTable,  bufferGraphics );

		_animation.insert( pair< size_t, animationDirection*>( direction, animDirection ) );
	}

	animationFrame	*actor::frameGet() {
		return _animation[ _positionDirection ]->frameGet( _action, _frameNumber );
	}

	bool actor::cycle() {
		
		healthUpdate();

		return true;	// Still alive
	}
}
