/* The Last Ninja Remixed
 *  
 * Copyright (C) 2008-2009 Robert Crossfield
 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * 
 * $Id: engine.cpp 15 2009-04-10 05:10:37Z segra $
 * 
 */

#include "stdafx.h"
#include "intro.h"
#include "introEclipse.h"

namespace NinjaRemixed {

	resources	*_Resources;

	engine::engine( size_t startLevelNumber ) : engineObject(0) {
		
		if( startLevelNumber < 1 || startLevelNumber > 6)
			_levelNumber	= 1;
		else
			_levelNumber	= startLevelNumber;

		_Resources		= new resources("data");
		_videoVirtual   = new virtualScreen(640, 400);
		_videoWindow	= new screen( 640, 400 );
		_hud			= new hud( this );
		_input			= new input();


		_quit			= false;

	}

	engine::~engine( ) {
		delete _Resources;
		delete _videoWindow;
		delete _videoVirtual;
		delete _hud;
		delete _input;
		delete _player;
		delete _level;

	}

	bool engine::cycle() {
		
		engineObject::cycle();
		return true;
	}

	void engine::main( ) {
		introEclipse	*IntroEclipse	= new introEclipse( this );
		intro			*Intro			= new intro( this );

		_player				= new actorPlayer( this, 0, 0, 0 );

		// Load the walk animations
		_player->actionLoad(0, 8);

		_level = new level( this, _levelNumber );

		while(!_quit) {

			if(!_input->cycle())
				_quit = true;

			// Draw the virtual screen
			_videoWindow->blit( _videoVirtual, 0, 0 );
			
			if(IntroEclipse) {

				if( !IntroEclipse->cycle() ) {
					delete IntroEclipse;
					IntroEclipse = 0;
				}
				continue;
			}
			// Playing the intro?
			if(Intro) {

				if( !Intro->cycle() ) {
					delete Intro;
					Intro = 0;
				}
				continue;
			}	// Intro

			_videoVirtual->clear();
		
			
			_hud->cycle();
			_level->cycle();
			cycle();
		}

	}

}
