/* The Last Ninja Remixed
 *  
 * Copyright (C) 2008-2009 Robert Crossfield
 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * 
 * $Id: animAction.cpp 12 2009-04-10 04:08:13Z segra $
 * 
 */

#include "..\stdafx.h"

namespace NinjaRemixed {


	animationAction::animationAction(word maxX, word maxY, word frameSize, const byte *actionGraphics, const word *actionIndex) {

		_actionGraphics = actionGraphics;
		_actionIndex	= actionIndex;
		_frameSize		= frameSize;

		_width  = maxX;
		_height = maxY;

		frameLoad( 0 );
	}

	animationAction::~animationAction() {
		map<size_t, animationFrame*>::iterator	framesIT;

		// Cleanup the allocated memory for each animationFrame
		for(framesIT = _frames.begin(); framesIT != _frames.end(); framesIT++)
			delete (*framesIT).second;

	}

	animationFrame *animationAction::frameLoad( size_t frameNumber ) {
		animationFrame		*frame;
		map<size_t, animationFrame*>::iterator	frameIT;

		const byte	*graphicCell;
		word		 graphicsIndex;

		// Check if the frame is already loaded
		frameIT = _frames.find( frameNumber );
		if(frameIT != _frames.end() )
			return (*frameIT).second;

		// Read the "animation cell number" from the animation cell listing
		graphicsIndex = readWord( _actionIndex + frameNumber );

		// Multiply the cell number by the size of a cell
		graphicsIndex *= _frameSize;		// 0x210: Size of cell (bytes)
		
		// Create the pointer to the cell
		graphicCell = _actionGraphics + graphicsIndex;

		// Create new animationFrame and insert it
		frame = new animationFrame( _width, _height, graphicCell, _frameSize );

		frame->paletteLoad( _Resources->_game + 0xA3A4 );
		frame->draw( );
		_frames.insert( pair<size_t, animationFrame*>( frameNumber,  frame ) );
		
		// Return the loaded frame
		return frameGet( frameNumber );
	}

	inline animationFrame	*animationAction::frameGet( size_t frameNumber ) {
		map<size_t, animationFrame*>::iterator	 frameIT;
		
		// Find the frame
		frameIT = _frames.find( frameNumber );
		
		// Did we just find the end instead of the frame?
		if( frameIT == _frames.end() )
			return frameLoad(frameNumber);
		
		// Return the animationFrame pointer
		return frameIT->second;
	}

}
