/* The Last Ninja Remixed
 *  
 * Copyright (C) 2008-2009 Robert Crossfield
 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * 
 * $Id: surface.h 12 2009-04-10 04:08:13Z segra $
 * 
 */


namespace NinjaRemixed {

#define colorsMax 16

	struct paletteColor {
		byte	red, green, blue;

		paletteColor() : red(0), green(0), blue(0) { }
	};

	class graphicSurface {
	private:
		//		Private Variables
		paletteColor		 _palette[colorsMax];		// Current Palette
		bool				 _scaled;
		//		Private Functions


	protected:
		//		Protected Variables
		const byte			*_buffer;					// Raw image data
		size_t				 _bufferSize;				// Raw image data size
		bool				 _loaded;					// Data decoded
		word				 _maxWidth, _maxHeight;		// Maximum X/Y
		SDL_Surface			*_surface;					// Current drawn surface

		//		Protected Functions
		virtual void		 decode(			size_t numBits, byte colorMask, const byte *buffer, size_t bufferSize);
		virtual void		 load(				const byte *buffer, size_t bufferSize) = 0;		// Decode buffer into internal _buffer
		void				 pixelDraw(			word  x, word y, dword paletteIndex );		// Plot a pixel on the SDL surface
		

	public:
		//		Public Functions

		//	Constructor/Destructor
							 graphicSurface(	word width, word height, const byte *buffer, size_t bufferSize );				// Create a surface
							~graphicSurface( );												// Free the surface

		void				 blit( graphicSurface	*Source, dword mask );					// Blit a surface 
		void				 blit( graphicSurface	*Source, word  destX, word destY, dword colorKey, dword colorKey2 = 0xFF);
		void				 blit( graphicSurface	*Source );	

		void				 unblit( graphicSurface	*Source, word destX, word destY, dword colorKey );

		word				 heightGet();
		word				 widthGet();

		// Palette Control
		void				 paletteLoad(		const byte  *buffer	);						// Load new palette
		paletteColor		 paletteColorGet(	byte id );									// Get a color from the palette
		void				 paletteColorSet(	byte id, byte red, byte green, byte blue );	// Set a color in the palette


		dword				*pixelGet( word x,	word y );

		// Surface Control
		void				 clear( byte color = 0xFF);											// Clear the screen
		void				 clear( word X, word Y, word sizeX, word sizeY, byte color = 0xFF);	// Clear a portion of screen

		SDL_Surface			*surfaceGet( );													// Get the SDL surface

		inline	bool		 scaleGet() { return _scaled; }									// Get Scale
		void				 scaleTo();														// Scale up this surface	

		
				void		 draw(  word X, word Y, bool scale = true );
		virtual void		 draw(	bool scale = true);										// Draw _buffer to the surface


	};

}
