/* The Last Ninja Remixed
 *  
 * Copyright (C) 2008-2009 Robert Crossfield
 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * 
 * $Id: textScreen.cpp 12 2009-04-10 04:08:13Z segra $
 * 
 */

#include "../stdafx.h"

namespace NinjaRemixed {
	_string::_string(unsigned int _x, unsigned int _y, string _dataString) {
		X = _x;
		Y = _y;
		dataString = _dataString;

	}

	graphicTextScreen::graphicTextScreen(const byte *buffer, size_t bufferSize) : graphicSurface( 320, 280, buffer, bufferSize ) {

	}
	
	graphicTextScreen::~graphicTextScreen() {
		vector<_string*>::iterator	 stringIT;

		for( stringIT = _strings.begin(); stringIT != _strings.end(); stringIT++)
			delete *stringIT;
		
	}

	void graphicTextScreen::draw(	) {
		_string						*stringCurrent;
		string::iterator			 charIT;
		vector<_string*>::iterator	 stringIT;

		unsigned int				 X, Y;

		if( !_loaded ) {
			load( _buffer, _bufferSize );

		}

		clear();

		for( stringIT = _strings.begin(); stringIT != _strings.end(); stringIT++) {
			stringCurrent = (*stringIT);

			X = stringCurrent->X * 8;
			Y = stringCurrent->Y;
		
			for(charIT = stringCurrent->dataString.begin(); charIT != stringCurrent->dataString.end(); charIT++) {
				graphicFont *Font = _Resources->resourceFont(*charIT);

				blit( Font , X, Y, 0xFF );
				X += 8;
			}
		}

		scaleTo();
	}

	void graphicTextScreen::load(	const byte *buffer, size_t bufferSize ) {
		unsigned int	  X, Y;
		byte			  byteNext = 0x00;
		stringstream	  tmpString;

		// Loop until end of strings marker found
		while(byteNext != 0xFE) {
			byteNext = 0;
			Y = *buffer++;
			X = *buffer++;		

			// Loop until end of string/strings marker found
			for( ; ; ) {
				byteNext = *buffer++;
			
				// Use the byte if not our marker
				if(byteNext != 0xFF && byteNext != 0xFE)
					tmpString << byteNext;
				else
					break;
			}
			
			// Add string to string container
			_strings.push_back( new _string(X, Y, tmpString.str()) );	
			tmpString.str("");
		}

		_loaded = true;
	}

}
