/* The Last Ninja Remixed
 *  
 * Copyright (C) 2008-2009 Robert Crossfield
 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * 
 * $Id: hud.cpp 12 2009-04-10 04:08:13Z segra $
 * 
 */

#include "stdafx.h"

namespace NinjaRemixed {

	hud::hud(engine *Engine) : graphicImage(320, 200, 0, 0), engineObject(Engine) {
		size_t	fileSize, finalSize = 0x1000;

		byte	*screenHUD	    = _Resources->resourceGet( 0x16, fileSize );
		screenHUD				= _Resources->resourceUnpack( screenHUD, finalSize );
		
		_lifePlayerImage		= 0;
		_lifeEnemyImage			= 0;

		// Decode into internal buffer
		load( screenHUD + 0x80, finalSize - 0x80 );

		// Load the palette then draw to internal buffer
		paletteLoad( screenHUD + 4 );
		draw();

		// Cleanup
		delete screenHUD;

		// Init values
		reset();
	}

	hud::~hud() {
		if( _lifePlayerImage)
			delete _lifePlayerImage;

		if(_lifeEnemyImage)
			delete _lifeEnemyImage;
	}

	bool hud::cycle() {
		
		_engine->videoVirtualGet()->blit( this );
		_engine->videoVirtualGet()->blit( _lifePlayerImage, 0, 0, 0xFF );
		_engine->videoVirtualGet()->blit( _lifeEnemyImage , 0, 0, 0xFF );
		

		return true;
	}

	void hud::enemyhealthUpdate( word damage ) {
		healthEnemyBar		*lifeBar;

		if( damage != _lifeEnemyDrawn) {

			// Increase enemy damage bar
			if((short int) _lifeEnemyDrawn < (short int) damage) {
				_lifeEnemyDrawn++;
			} else
				return;

			lifeBar = new healthEnemyBar( _lifeEnemyDrawn );
			lifeBar->draw(0, 0, true);

			// Load the next damage bar
			_lifeEnemyImage->blit( lifeBar, lifeBar->_X, lifeBar->_Y , 0xFF, 0x00 );

			delete lifeBar;
		}
	}

	void hud::playerHealthUpdate( word life ) {
		word				 healthDraw = 0;
		healthPlayerBar		*lifeBar;
		bool				 draw		= false;		// Does the bar get drawn, or do we draw black over it

		if( life != _lifePlayerDrawn) {
			healthDraw = _lifePlayerDrawn;

			// Increase or decrease the health bar
			if((short int) _lifePlayerDrawn < (short int) life) {
				draw = true;
				healthDraw++;
				_lifePlayerDrawn++;

			} else {
				_lifePlayerDrawn--;
			}

			lifeBar = new healthPlayerBar( healthDraw );
			lifeBar->draw(0, 0, true);

			// Load/Unload the next health bar
			if(draw)
				_lifePlayerImage->blit( lifeBar, lifeBar->_X, lifeBar->_Y , 0xFF, 0x00 );
			else
				_lifePlayerImage->unblit( lifeBar, lifeBar->_X, lifeBar->_Y, 0xFF);
				
			delete lifeBar;
		}

	}

	void hud::reset() {

		// Cleanup current player health image
		if(_lifePlayerImage)
			delete _lifePlayerImage;

		if(_lifeEnemyImage)
			delete _lifeEnemyImage;

		_lifePlayerDrawn	= 0xFFFF;								// No life drawn
		_lifePlayerImage	= new graphicImage(640,480, 0, 0);		// HACK: we need to define the resolution somewhere
		_lifePlayerImage->clear(0xFF);


		_lifeEnemyDrawn		= 0x00;									// Enemy Death Bar, None drawn
		_lifeEnemyImage		= new graphicImage(640,480, 0, 0);
		_lifeEnemyImage->clear(0xFF);
	}

	void hud::roomChange() {
		_lifeEnemyDrawn		= 0x00;
		_lifeEnemyImage->clear(0xFF);

	}

}
