/* The Last Ninja Remixed
 *  
 * Copyright (C) 2008-2009 Robert Crossfield
 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * 
 * $Id: intro.cpp 13 2009-04-10 05:02:33Z segra $
 * 
 */

#include "stdafx.h"
#include "intro.h"

namespace NinjaRemixed {

	intro::intro(engine	*engine) : graphicImage(320, 200, 0, 0), engineObject(engine) {
		size_t				 fileSize, finalSize = 0x5000;
		byte				*system3Stream	= _Resources->resourceGet( "System3", fileSize );
		system3Stream						= _Resources->resourceUnpack( system3Stream, finalSize );
	
		_stage		  = 0;
		_timer		  = 100;
	
		// Load into internal buffer
		load( system3Stream + 0x80 , finalSize - 0x80);

		// Load the palette
		paletteLoad( system3Stream + 4 );

		// Draw to the SDL surface
		draw();

		// Cleanup
		delete system3Stream;
	}

	intro::~intro() {
	
	}

	bool intro::cycle() {
		static word		colorFadeCurrent = 0x888;

		// This will skip the intro
		//if(_stage==0)
		//	_stage = 7;
		switch(_stage) {
			case 0:	// System 3 Fade in
				colorFadeCurrent -= 0x111;

				_engine->videoVirtualGet()->fade( this, colorFadeCurrent );

				if(colorFadeCurrent == 0)
					_stage++;
				return true;

			case 1:	// Pause
			case 3:
			case 5:
				Sleep(10);
				_timer--;

				if(_timer == 0)
					_stage++;

				return true;

			case 2:	// System 3 Fade out
				colorFadeCurrent += 0x111;

				_engine->videoVirtualGet()->fade( this, colorFadeCurrent );

				if(colorFadeCurrent == 0x888) {
					_stage++;
					_timer		  = 100;
				}

				return true;
			
			case 4:	// HUD Fade in
				colorFadeCurrent -= 0x111;

				_engine->videoVirtualGet()->blit( _Resources->resourceText( 6 ) );
				_engine->videoVirtualGet()->fade( _engine->hudGet(), colorFadeCurrent );
				
				if(colorFadeCurrent == 0) {
					_stage++;
					_timer		  = 100;
				}

				return true;

			case 6:	{// Press any button to play
				static dword mask = 0x00;
				static graphicTextScreen *screen = _Resources->resourceText( 7 );

				//_hud->draw( mask );
				//screen->draw(  );

				mask <<= 1;
				mask += 1;

				//_engine->videoVirtualGet()->blit( _Resources->resourceText( 6 ) );
				//_engine->videoVirtualGet()->pixelMask( mask );
				
				_stage++;
				return true;
					}
			case 7:
				return false;
		}

		return false;
	}


}
