/* The Last Ninja Remixed
 *  
 * Copyright (C) 2008-2009 Robert Crossfield
 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * 
 * $Id: introEclipse.cpp 14 2009-04-10 05:10:15Z segra $
 * 
 */

#include "stdafx.h"
#include "introEclipse.h"

namespace NinjaRemixed {

	introEclipse::introEclipse(engine	*engine) : graphicImage(320, 200, 0, 0), engineObject(engine) {
		size_t				 fileSize, finalSize = 0x5000;
		byte				*eclipseStream	= _Resources->resourceGet( "Eclipse", fileSize );
		eclipseStream						= _Resources->resourceUnpack( eclipseStream, finalSize );
	
		_stage		  = 0;
		_timer		  = 100;
	
		// Load into internal buffer
		load( eclipseStream + 0x80 , finalSize - 0x80);

		// Load the palette
		paletteLoad( eclipseStream + 4 );

		// Draw to the SDL surface
		draw();

		// Cleanup
		delete eclipseStream;
	}

	introEclipse::~introEclipse() {
	
	}

	bool introEclipse::cycle() {
		static word		colorFadeCurrent = 0x888;

		// This will skip the intro
		//if(_stage==0)
		//	_stage = 7;
		switch(_stage) {
			case 0:	// Eclipse Fade in
				colorFadeCurrent -= 0x111;

				_engine->videoVirtualGet()->fade( this, colorFadeCurrent );

				if(colorFadeCurrent == 0)
					_stage++;
				return true;

			case 1:	// Pause
			case 3:
			case 5:
				Sleep(10);
				_timer--;

				if(_timer == 0)
					_stage++;

				return true;

			case 2:	// Eclipse Fade out
				colorFadeCurrent += 0x111;

				_engine->videoVirtualGet()->fade( this, colorFadeCurrent );

				if(colorFadeCurrent == 0x888) {
					_stage++;
					_timer		  = 100;
				}

				return true;
			
			case 6:
				return false;
		}

		return false;
	}


}
