/* The Last Ninja Remixed
 *  
 * Copyright (C) 2008-2009 Robert Crossfield
 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * 
 * $Id: level.cpp 13 2009-04-10 05:02:33Z segra $
 * 
 */

#include "stdafx.h"


namespace NinjaRemixed {

	level::level(engine *engine, size_t levelNumber) : engineObject(engine) {
		size_t	 fileSize, finalSize = 0x3E8;
		byte	*loadScreen;

		// Set internals
		_loadTimer	 = 0x100;
		_levelNumber = levelNumber;		
		_roomNumber  = 0x01;
		
		// This is for multiplication purposes 
		levelNumber--;

		// Load the eyes / level names
		loadScreen		= _Resources->resourceGet	( 0x17, fileSize );
		loadScreen		= _Resources->resourceUnpack(loadScreen, finalSize, true);

		// Load the eyes
		_loadScreen  = new graphicImage( 240, 136, loadScreen, 0x29b8);
		_loadScreen->paletteLoad( _Resources->_game + 0xA3A4 );

		// Load the screen name
		_loadScreenName = new graphicImage( 240, 40, (loadScreen + 0x29b8) + (0x1248 * levelNumber), 0x1248);
		_loadScreenName->paletteLoad( _Resources->_game + 0xA3A4 );

		// Eyes lower for lvl1
		if(_levelNumber == 1)
			_loadScreen->draw( 0, 17 );
		else
			_loadScreen->draw( );

		_loadScreenName->draw( );
	
		// No name for level 1
		if(_levelNumber > 1)
			_loadScreen->blit(_loadScreenName, 0, 0x60, 0xFF);
		
		// Cleanup Unpacked data
		delete loadScreen;

		// 0x9A92:  levelData table offset, 0x16 Size of each entry
		loadData( (_Resources->_game + 0x9A92) + (levelNumber * 0x16));
		
		finalSize = 0x3E8;

		// Load the level data
		_levelData.UnpackedData = _Resources->resourceGet   ( _levelData.levelID + 1,   fileSize );
		_levelData.UnpackedData = _Resources->resourceUnpack( _levelData.UnpackedData,  finalSize );

		// Load the level backgrounds
		_levelBackgrounds		= _Resources->resourceGet   ( _levelData.levelID,		_levelBackgroundSize );

		// Load the first room
		_room = roomLoad( _roomNumber );

		// Set the players starting location
		//	0x9C1E: Offset into 'Game' object, 0x5: Size of levelStart struct
		_levelStart = (levelStart*) (_Resources->_game + 0x9C1E + (levelNumber * 0x5));
		
		// Set the player
		_engine->playerGet()->set( _levelStart ) ;
	}

	level::~level() {
		vector<room*>::iterator	roomIT;

		if(_loadScreen)
			delete _loadScreen;

		if(_loadScreenName)
			delete _loadScreenName;

		if(_levelBackgrounds)
			delete _levelBackgrounds;

		if(_levelData.UnpackedData)
			delete _levelData.UnpackedData;

		for(roomIT = _rooms.begin(); roomIT != _rooms.end(); roomIT++)
			delete *roomIT;

	}

	bool level::cycle() {
	
		if( _loadTimer ) {
			_engine->videoVirtualGet()->blit( _loadScreen );
			_loadTimer--;

			if(_engine->inputGet()->inputGet() == inputBUTTON)
				_loadTimer = 0;

			return true;
		}

		if(_engine->inputGet()->inputGet() == inputBUTTON) {
			roomLoad( _roomNumber + 1 );
			roomChange( _roomNumber + 1);
		}

		
		_room->cycle();

		return true;
	}

	void level::loadData( byte *levelPtr ) {

		_levelData.levelID			= readWord ( levelPtr );
		_levelData.roomTrackSectors = readDWord( levelPtr + 0x02 );
		_levelData.roomTrees		= readDWord( levelPtr + 0x0A );
		_levelData.roomObjects		= readDWord( levelPtr + 0x06 );
		_levelData.roomPtr			= readDWord( levelPtr + 0x0E );
		_levelData.roomPalette		= readDWord( levelPtr + 0x12 );

	}

	room *level::roomFind( size_t roomNumber ) {
		vector<room*>::iterator	roomIT;

		for(roomIT = _rooms.begin(); roomIT != _rooms.end(); roomIT++)
			// Is this the room?
			if( (*roomIT)->roomNumberGet() == roomNumber)	
				return (*roomIT);


		return 0;
	}

	void level::roomChange( size_t roomNumber ) {
		room *Room = roomFind( roomNumber);

		// Not loaded?
		if(!Room)
			return;

		// Update the GUI
		_engine->hudGet()->roomChange();

		// Alter the players position based on the entering screen positions
		_engine->playerGet()->roomEnter( _roomNumber, roomNumber );

		_room = Room;
		_roomNumber = roomNumber;
	}

	room *level::roomLoad( size_t roomNumber ) {
		room *Room = roomFind( roomNumber);
		
		if(Room)
			return Room;

		Room = new room( _engine, roomNumber, _levelNumber, _levelBackgrounds, &_levelData);

		_rooms.push_back( Room );

		return Room;
	}

}
