/* The Last Ninja Remixed
 *  
 * Copyright (C) 2008-2009 Robert Crossfield
 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * 
 * $Id: overlay.cpp 13 2009-04-10 05:02:33Z segra $
 * 
 */

#include "stdafx.h"

namespace NinjaRemixed {

	roomOverlay::roomOverlay(engine *Engine, room *Room, word d3, byte *buffer, size_t bufferSize, word maxX, word maxY ) : graphicImage(maxX * 16, maxY , 0, 0), engineObject(Engine) {
		_d3 = d3;

		// Load our palette
		paletteLoad( Room->levelDataGet()->UnpackedData + (Room->levelDataGet()->roomPalette - Room->levelDataGet()->roomTrees) );

		_positionX = readWord( buffer ) * 16;
		_positionY = readWord( buffer + 2 );
		_maxHeight = readWord( buffer + 4 );
		_maxWidth  = readWord( buffer + 6 );

		load( buffer + (4 * sizeof(word)), bufferSize - (4 * sizeof(word)) );

		draw(true);
	}

	void roomOverlay::load( const byte *buffer, size_t bufferSize ) {
		decode( 4, 0xD, buffer, bufferSize);
	}

	void roomOverlay::draw( bool scale ) {
		word	 baseX = 0, X = 0, Y = 0;
		byte	*bufferCurrent	= (byte*) _buffer;

		if(!_loaded && bufferCurrent) {
			// Load the image
			load( _buffer, _bufferSize );
			
			bufferCurrent = (byte*) _buffer;
		}

		clear();

		int count = 0;
		Y=0;

		while( bufferCurrent < ( byte*) (_buffer + _bufferSize ) ) {
			
			pixelDraw( X, Y,  *bufferCurrent );

			bufferCurrent++;
			X++;

			if(count++ == 15) {

				count = 0;
				Y++;
				X = baseX;

				if(Y == _maxHeight) { 
					Y = 0;
					baseX += 16;
				}
			}

		}	// while( bufferCurrent < ( byte*) (_buffer + _bufferSize ) )

		if(scale)
			// Scale it up
			scaleTo();
	}
}
