/* The Last Ninja Remixed
 *  
 * Copyright (C) 2008-2009 Robert Crossfield
 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * 
 * $Id: resources.cpp 12 2009-04-10 04:08:13Z segra $
 * 
 */

#include "stdafx.h"


// LOB Size unpack table
word sizeTable[] = { 0x00,
					 0x800,
					 0x3C00,
					 0x3C00,
					 0x00,
					 0x00,
					 0x300,
					 0x0,
					 0x00 };

namespace NinjaRemixed {

	resources::resources(string pathFiles) {
		byte			*bufferPtr;

		// Set data directory
		_dataDirectory = pathFiles;
		if(_dataDirectory[ _dataDirectory.size() ] != '\\')
			_dataDirectory.append("\\");

		// Load the main game file
		_game = fileRead( "Game", _gameSize );

		// Decode the main game file
		bufferPtr = _game = LOB_Decode( _game, 0x5000, _gameSize );

		// stringsLoad uses _Resources pointer to obtain fonts
		_Resources = this;

		// Load Fonts/Strings
		fontsLoad();
		stringsLoad();
	}

	resources::~resources() {
		vector<graphicFont*>::iterator			fontIT;
		vector<graphicTextScreen*>::iterator	textIT;

		for( fontIT = _fonts.begin(); fontIT != _fonts.end(); fontIT++)
			delete *fontIT;

		for( textIT = _text.begin(); textIT != _text.end(); textIT++)
			delete *textIT;

	}

	dword *resources::LOB_Decode_Chunk(byte *stream, dword *streamDword, unsigned int size, unsigned int sizeTab, unsigned int &finalSize) {
		byte	*streamDestination;
		byte	*streamSrc = (byte*) streamDword;
		byte	*streamDest, *streamDest2, *streamDest3;

		int		 count;
		dword	 destSize, srcSize;

		dword	*dest, *src;

		destSize = readDWord(streamDword)  &  0xFFFFFF;

		finalSize = destSize + size;

		streamDestination = new byte[finalSize];

		// Copy the data to the new buffer
		if(stream[0] == 0x08)
			for(dword i = 0; i < (destSize + size); i++)
				streamDestination[i] = streamSrc[i];

		streamDest = streamDestination;

		srcSize = readDWord(streamDword+1);
		srcSize += 8;
		srcSize += sizeTab;

		streamSrc += srcSize;

		streamDest += destSize;
		if(stream[0] == 0x08)
			streamDest += size;

		count = srcSize;

		// Set DWord Pointers
		src  = (dword*) streamSrc;	dest = (dword*) streamDest;

		do {
			// Decrease main pointers
			streamSrc -= 8;		streamDest -= 8;
			// Move 2 DWORDS
			*--dest = *--src;		*--dest = *--src;
			count -= 8;
		} while( count >= 0);

		streamDest -= count;
		streamDest2 = streamDestination;

		if(streamDest[0] == 6) {

			// destSize
			streamDest+=4;

			// 
			streamDest+=4;

			byte  d = 0x80;
			dword d1 = 0, d2, d3;

			while(streamDest2 < (streamDestination + destSize)) {

				d1 = d;
				if( (d += d) == 0 ) {
					d = *streamDest++;
					d1 = d;
					d += d + 1;

				}

				d1 += d1;
				if ( d1 < 0x100) {
					// Chunk move
					d3 = 0;
					d3 = *streamDest++;
					
					d2 = d3 & 0xF;		// Bytes to move (max 16)

					d3 <<= 4;
					d3 &= ~0xFF;	// Everything but the lsbyte (least significant)
					d3 += *streamDest++;

					streamDest3 =(byte*) streamDest2;		// streamDest2 = A1
					streamDest3 -= d3;

					*streamDest2++ = *streamDest3++;
					*streamDest2++ = *streamDest3++;

					count = d2;
					while(count >= 0) {
						*streamDest2++ = *streamDest3++;
						count--;
					}
				}	else // Chunk move
					*streamDest2++ = *streamDest++;
			}	// while(streamDest2 < (streamDestination + destSize))
		}	else 
			// Mode 8
			if( streamDest[0] == 8) {
				
			// streamDest = A0
			// streamDest2 = A1

			// destSize
			streamDest+=4;
			streamDest+=4;

			streamDest3 = streamDest2 + destSize;		// streamDest3 = A5
			
			// A0/A1 Swapped here
			// streamDest = A1
			// streamDest2 = A0

			int	   d0 = 0x00;
			dword  d1 = 0x28;
			byte  *a3, *a2, *a4;
			word  *streamDest4;

			a4 = streamDest;

			while(streamDest2 < streamDest3) {
				if( d0-- <= 0 ) {

					streamDest = (byte*) a4;	
					d0 = 0x13;

					a2 = streamDest;
					a2 += d1;

					a3 = a2;
					a3 += d1;

					a4 = a3;
					a4 += d1;
				}

				streamDest4 = (word*) streamDest2;

				*streamDest4++ = *((word*)streamDest);
				*streamDest4++ = *((word*)a2);
				*streamDest4++ = *((word*)a3);
				*streamDest4++ = *((word*)a4);

				streamDest+=2;
				a2+=2;
				a3+=2;
				a4+=2;

				streamDest2 = (byte*) streamDest4;
			}
		}

		return (dword*) streamDestination;
	}

	byte *resources::LOB_Decode(byte *stream, unsigned int size, unsigned int &finalSize, bool cleanup) {
		dword *streamDword = (dword*) stream;
		dword *streamNew;
		dword xLOB;
		byte  count;
		word  index;

		xLOB = readDWord(streamDword++);

		count = xLOB >> 24;

		while(count) {

			index = *((byte*)streamDword);

			// 
			streamNew = LOB_Decode_Chunk( stream, streamDword, size, sizeTable[index-1], finalSize );

			// Do we delete the memory? or is it handled elsewhere
			if(cleanup)
				delete stream;

			// Change our pointers to the new stream
			stream = (byte*) streamNew;
			streamDword = (dword*) streamNew;

			count--;
		}

		return stream;
	}

	byte		*resources::fileRead( string file, size_t	&fileSize ) {
		stringstream	 filePathFinal;
		ifstream		*fileStream;
		byte			*fileBuffer = 0;

		// Build the file path
		filePathFinal << _dataDirectory;
		filePathFinal << file;

		// Attempt to open the file
		fileStream = new ifstream ( filePathFinal.str().c_str(), ios::binary );
		if(fileStream->is_open() == false)
			return 0;

		// Get file size
		fileStream->seekg(0, ios::end );
		fileSize = fileStream->tellg();
		fileStream->seekg( ios::beg );

		// Allocate buffer, and read the file into it
		fileBuffer = new byte[ fileSize ];
		if(fileStream->read( (char*) fileBuffer, fileSize ) == false)
			return 0;

		// Close the stream
		fileStream->close();

		// All done ;)
		return fileBuffer;
	}

	void	resources::fontsLoad() {
		byte			*bufferPtr = _game;
		graphicFont		*font;

		// Move forward to fonts
		bufferPtr += 0xC368;

		while( bufferPtr < (_game + (0xC368 + 0x600)) ) {
			font = new graphicFont( bufferPtr, 0x18 );

			// Gimme the palette ;)
			font->paletteLoad(  _game + 0xA3A4 );

			// No scaling, this is done when the fonts are on the text screen
			font->draw(  false  );

			_fonts.push_back(font);

			// Next!
			bufferPtr += 0x18;
		}
	}

	graphicFont			*resources::resourceFont( byte character ) {
		character &= 0x3F;

		return _fonts[ character ];
	}

	graphicTextScreen	*resources::resourceText( size_t ID ) {
		if(	ID >= _text.size() )
			return 0;

		return _text[ ID ];
	}

	byte		*resources::resourceGet( unsigned int ID, size_t &fileSize ) {
		byte			*byteStream;
		stringstream	 fileName;
		stringstream	 filePath;
		size_t			 countRetry;
		char			*directorys[] = { "",
										  "Disk1\\",
										  "Disk2\\",
										  "Disk3\\",
										  0 };

		fileName << "$000000";

		if(ID < 0x10)
			fileName << "0";

		fileName << hex << ID;

		for(countRetry = 0; countRetry < 4; countRetry++) {
			
			byteStream = fileRead( directorys[countRetry] + fileName.str(), fileSize );
			if(byteStream)
				break;

		}

		return byteStream;
	}

	byte		*resources::resourceGet( string fileName, size_t &fileSize ) {

		return fileRead( fileName, fileSize );
	}

	byte		*resources::resourceUnpack( byte *LOBstream, size_t	&size, bool cleanup ) {

		// Ensure its a LOB first ;)
		if(LOBstream && LOBstream[1] == 'L' && LOBstream[2] == 'O' && LOBstream[3] == 'B')
			return LOB_Decode( LOBstream, size, size, cleanup );
		
		return 0;
	}

	void		 resources::stringLoad( byte *stringPtr ) {
		graphicTextScreen *textScreen = new graphicTextScreen( stringPtr, 0 );
		
		// Load the string and draw it to the surface
		textScreen->draw();

		_text.push_back( textScreen );
	}

	void		 resources::stringsLoad() {
		stringLoad(_game + 0x9CCF);		// Insert Disk
		stringLoad(_game + 0x9D00);		// Lives Remaining
		stringLoad(_game + 0x9D22);		// Save Game?
		stringLoad(_game + 0x9D60);		// Load Game?
		stringLoad(_game + 0x9DA0);		// Game Over
		stringLoad(_game + 0x9DF2);		// Coming Soon
		stringLoad(_game + 0x9FB0);		// Intro Screen 1
		stringLoad(_game + 0xA0D0);		// Press any key
	}

}
