/* The Last Ninja Remixed
 *  
 * Copyright (C) 2008-2009 Robert Crossfield
 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * 
 * $Id: room.cpp 13 2009-04-10 05:02:33Z segra $
 * 
 */

#include "stdafx.h"

namespace NinjaRemixed {
	struct levelData;


	room::room(engine *engine, word roomNumber, word levelNumber, byte *levelImages, levelData *leveldata) : graphicImage( 240, 144, 0, 0 ), engineObject( engine ) {

		_enemy		= 0;
		_roomItem	= 0;

		_levelData	= leveldata;
		_roomNumber = roomNumber;
		_levelNumber = levelNumber;

		backdropLoad(levelImages);
		
		overlaysLoad();

		itemLoad();

		//riverLoad();

		draw( );

		enemyLoad();
	}

	room::~room() {
		vector<actor*>::iterator	actorIT;

		// Cleanup any actors
		if(_enemy)
			delete _enemy;

		if(_roomItem)
			delete _roomItem;
	}


	void room::backdropLoad(byte *roomImage) {
		dword	 offset = 0;
		size_t	 size;
		byte	*roomSizes;

		// Get pointer to the room sectors table
		roomSizes = ( _Resources->_game + (_levelData->roomTrackSectors - 0x101C) + 0x8);

		// Count the number of sectors for each room
		for(word count = 0; count < _roomNumber-1; count++) {
			offset	  += readWord(roomSizes); 
			roomSizes += 2;
		}
		
		// roomSizes now points to the number of sectors for the current room 

		// Calculate the offset for the current room
		offset *= 0x200; // 0x200 = Size of sector
		size	= 0x3E8;

		// Unpack the image (the original roomImage value is not cleaned up because its handled by the 'level' class
		roomImage = _Resources->resourceUnpack( roomImage + offset, size, false );
		
		// Decode the image into our _buffer
		load( roomImage, size );

		// Load our palette
		paletteLoad( _levelData->UnpackedData + (_levelData->roomPalette - _levelData->roomTrees) );

		// Cleanup
		delete roomImage;
	}

	bool room::cycle() {
		vector<roomOverlay*>::iterator	overlayIT;

		// Draw the background
		_engine->videoVirtualGet()->blit( this );
		
		// Probably check if this returns false (dead)
		_engine->playerGet()->cycle();
		
		for(overlayIT = _overlays.begin(); overlayIT != _overlays.end(); overlayIT++) {
			_engine->videoVirtualGet()->blit( *overlayIT, (*overlayIT)->_positionX, (*overlayIT)->_positionY, 0x00e0e0e0 );
		}

		if( _roomItem ) {
			_engine->videoVirtualGet()->blit( _roomItem, _roomItem->posXGet(), _roomItem->posYGet(), 0x00e0e0e0 );
		}

		// Draw the player
		_engine->videoVirtualGet()->blit( _engine->playerGet()->frameGet(), _engine->playerGet()->positionXGet(), _engine->playerGet()->positionYGet(), 0x00e0e0e0);

		if(_enemy) {
			_enemy->cycle();
			_engine->videoVirtualGet()->blit( _enemy->frameGet(), _enemy->positionXGet(), _enemy->positionYGet(), 0x00e0e0e0 );
		}

		return true;
	}

	void room::itemLoad() {
		word	 roomNumber		= _roomNumber;
		byte	*objectData	= _Resources->_game + (_levelData->roomObjects) - 0x101c;
		byte	*objectDat = objectData;
		word	 roomNoCurrent;
		word	 bufferSize, sizeX, sizeY;


		while( (roomNoCurrent = readWord( (word*) objectData )) != 0xFFFF ) {
			objectData += 2;
			sizeX = *(objectData + 4);
			sizeY = *(objectData + 5);

			bufferSize = sizeY;
			bufferSize <<= 3;

			bufferSize *= sizeX;
			bufferSize += 6;

			// Only 1 item ever exists in a room, so once its found we can leave
			if( roomNoCurrent == roomNumber ) {
				_roomItem = new item( objectData, bufferSize, *(objectData + 3), sizeY );

				break;
			}
			
			objectData += bufferSize;
		}
	}

	void room::overlaysLoad() {
		dword	levelTrees	  = (dword) _levelData->UnpackedData;
		dword	levelTreesEnd = (dword) _levelData->UnpackedData + (_levelData->roomPtr - _levelData->roomTrees);
		byte	counter		  = 100;
		word	levelNumber, roomNumber, dataSize, d3, maxX, maxY;
		word	d7;

		while( (levelTrees < levelTreesEnd) ) {
		
			roomNumber	= readWord( (word*) levelTrees );
			dataSize	= readWord( (word*) (levelTrees + sizeof(word)) );
			d3			= readWord( (word*) (levelTrees + ( 2 * sizeof(word))) );
			maxY		= readWord( (word*) (levelTrees + ( 3 * sizeof(word))) );
			maxX		= readWord( (word*) (levelTrees + ( 4 * sizeof(word))) );
			
			// Calculate data size
			d7 = maxY;
			d7 <<= 3;
			d7 *= maxX;
			d7 += 10;

			if( roomNumber == _roomNumber ) {
				roomOverlay *Overlay = new roomOverlay(_engine, this, d3 + maxX, (byte*) (levelTrees + 2), d7 - 2, maxX, maxY);

				_overlays.push_back( Overlay );
			}

			if(!(--counter))
				break;

			// Skip over this overlays data

			levelTrees += d7;
		}
		
		roomNumber = _roomNumber;
		roomNumber++;
		levelNumber = _levelNumber;
		levelNumber--;
		levelNumber += levelNumber;
		levelNumber += levelNumber;

		byte	*levelTable = (byte*) (_Resources->_game + 0x6106) - 0x101C;
		dword	 overlay;

		levelTable += levelNumber;

		if( (overlay = readDWord( (byte*) levelTable)) == 0xFFFFFFFF )
			return;

		size_t countOverlays = 0;

		while(readWord( (byte*) (_Resources->_game + overlay) - 0x101C) != 0xFFFF) {
			
			if( readWord( (byte*) (_Resources->_game + overlay) - 0x101C) == _roomNumber ) {
				
				word d2, d3, d4;
				d2 = readWord( (byte*) (_Resources->_game + overlay + 2) - 0x101C);
				
				d2 += d2;
				d3 = d2;
				d3 += d2;
				d3 += d2;

				d4 = readWord( (byte*) (_Resources->_game + overlay + 4) - 0x101C);
				countOverlays++;
			}

			overlay += 6;

		}
	}

	void room::enemyLoad() {
		byte X, Y, Weapon;

		// Number of rooms in this level
		word	 levelRooms		=	readWord(  (word*) (_Resources->_game + 0x99A0 + (_levelData->levelID)) );
		dword	 levelEnemyPtr	=   readDWord((dword*) (_Resources->_game + 0x9788 + (_levelData->levelID * 2))) - 0x101C;
		
		byte	*levelEnemys    =	_Resources->_game + levelEnemyPtr + (_roomNumber - 1);

		// Room has enemy?
		if(!(*levelEnemys))
			return;				// No!

		// For some reason all these values are stored seperatly,
		// with a gap the size of 1 byte per number of rooms 
		
		levelEnemys += levelRooms;
		X = *levelEnemys;

		levelEnemys += levelRooms;
		Y = *levelEnemys;

		levelEnemys += levelRooms;
		Weapon = *levelEnemys;

		// Enemy found
		_enemy = new actorEnemy(_engine, X, Y, 0, Weapon);
	}

}
